#!/usr/bin/env bash
fname="$1"
print_header () {
cat <<EOF
digraph state_space {
	rankdir=TB;
	size=5;
	node [shape=circle, color=black, width=1];
EOF
}
print_footer () {
cat <<EOF
}
EOF
}

n=0
fpga_fname=$fname"_fpga"
ext_fname=$fname"_ext"
fpga_dot_fname=$fname"_fpga.dot"
ext_dot_fname=$fname"_ext.dot"
print_header > "$fpga_dot_fname"
print_header > "$ext_dot_fname"
while read line ; do
	((n=n+1))
	fpga_pins=""
	ext_pins=""
	old_fpga_hex_val=$fpga_hex_val
	old_ext_hex_val=$ext_hex_val
	for i in `seq 0 63` ; do
		pin=${line:$((i*2)):2}
		mode=${pin:1:1} # mode and val bits are reversed, see ca_state.c, written high to low now
		val=${pin:0:1}
		if [ "$mode" -eq 0 ] ; then
			fpga_pins=$fpga_pins$val
		elif [ "$mode" -eq 1 ] ; then
			ext_pins=$ext_pins$val
		else
			echo "somethingwrong" #should break the ps
		fi
	done
	fpga_hex_val=`echo "scale=20; obase=16; ibase=2; $fpga_pins" | bc`
	ext_hex_val=`echo "scale=20; obase=16; ibase=2; $ext_pins" | bc`
	if [ "$n" -gt 1  ] ; then
		echo -e "\t\"$old_fpga_hex_val\" -> \"$fpga_hex_val\";" >> "$fpga_dot_fname"
		echo -e "\t\"$old_ext_hex_val\" -> \"$ext_hex_val\";" >> "$ext_dot_fname"
	fi
done
print_footer >> "$fpga_dot_fname"
print_footer >> "$ext_dot_fname"
#fpga_buf=`cat $fpga_dot_fname`
#ext_buf=`cat $ext_dot_fname`

# first argument current dot-filename
fix_labels () {
	file_buf="`grep '\->' "$1" | sort | uniq -c`"
	fixed_file_buf="`cat "$1"`"
	echo "$fixed_file_buf" | awk '!_[$0]++' > "$1"
	while read -r line ; do
		NUM=${line%% *}
		TXT=${line#* }
		TXT=${TXT%%;*}
		if [ "$NUM" -ne 1 ] ; then
#			echo "NUM="$NUM
#			echo "TXT="$TXT
			sed -i -e "s/$TXT/$TXT \[ label = $NUM \]/g" "$1"
		fi
	done <<< "$file_buf"
}
fix_labels "$fpga_dot_fname"
fix_labels "$ext_dot_fname"
fix_header () {
	node="`grep -m 1 '\->' "$1" | sed -e 's/^[^"]*"\([^"]\+\)".*$/\1/'`"
	#echo "$node"
	replace=`grep 'node' "$1" | sed -e 's/^\t//' |sed -e 's/\[/\\\[/' | sed -e 's/\]/\\\]/'`
	#echo "$replace"
	sed -i -e "s/$replace/node \[shape=doublecircle, color=red, width=2\]; \"$node\";\n\t$replace/" "$1"
}

fix_header "$fpga_dot_fname"
fix_header "$ext_dot_fname"

fpga_eps_fname=$fpga_fname".pdf"
ext_eps_fname=$ext_fname".pdf"
neato -Tpdf "$fpga_dot_fname" -o"$fpga_eps_fname"
neato -Tpdf "$ext_dot_fname" -o"$ext_eps_fname"
#epstopdf $fpga_eps_fname
#epstopdf $ext_eps_fname
#grep -m 1 '\->' test_fpga.dot | sed -e 's/^[^"]*"\([^"]\+\)".*$/\1/'
