library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.cpu_package.all;

entity alu is
	port	(
			alu_in_a : in std_logic_vector(DDATA_BUS - 1 downto 0);
			alu_in_b : in std_logic_vector(DDATA_BUS - 1 downto 0);
			alu_func : in std_logic_vector(FUNC_BUS - 1 downto 0);

			alu_out : out std_logic_vector(DDATA_BUS - 1 downto 0);
			alu_status : out std_logic_vector(STATUS_BUS - 1 downto 0)
		);
end alu;

architecture alu_arch of alu is
	signal alu_out_i : std_logic_vector(alu_out'range);
begin
alu_status <= (others => '0');
alu_out <= alu_out_i;

process (alu_in_a, alu_in_b, alu_func)
	begin
		case alu_func is
		when ALU_FUNC_NOT =>
			alu_out_i <= not alu_in_a;

		when ALU_FUNC_AND =>
			alu_out_i <= alu_in_a and alu_in_b;

		when ALU_FUNC_OR =>
			alu_out_i <= alu_in_a or alu_in_b;

		when ALU_FUNC_XOR =>
			alu_out_i <= alu_in_a xor alu_in_b;

		when ALU_FUNC_NOP =>
			alu_out_i <= alu_in_a;

		when others =>
			alu_out_i <= (others => '0');
		end case;
	end process;
end alu_arch;
