----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:41:57 11/09/2011 
-- Design Name: 
-- Module Name:    ca_cpu - ca_cpu_arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.cpu_package.all;

entity ca_cpu is
	port (
			clk : in std_logic;
			rst : in std_logic;
			-- hold reset high until you want to start the cores
			core_rst : in std_logic;
			-- memory
			mem_w_address_array : in mem_w_address_a(0 to N_CORES - 1);
			mem_w_data_array : in mem_w_data_a(0 to N_CORES - 1);
			mem_w_enable_array : in mem_w_enable_a(0 to N_CORES - 1);
			mem_w_out_array : out mem_w_out_a(0 to N_CORES - 1);
			-- old state, this should be registered
			conf_reg_i : in std_logic_vector(CONF_BUS - 1 downto 0);
			-- new state, registered output
			conf_reg_o : out std_logic_vector(CONF_BUS - 1 downto 0);
			-- done status flag, means that all cores are finished
			done_o : out std_logic
			--cpu_out_o : out cpu_out_a(0 to N_CORES - 1)
			);
end ca_cpu;

architecture ca_cpu_arch of ca_cpu is

--type instances
signal core_rst_s : std_logic_vector(N_CORES - 1 downto 0);
--signal core_rst_s : rst_a(0 to N_CORES - 1);
signal mem_w_address_s : mem_w_address_a(0 to N_CORES - 1);
signal mem_w_data_s : mem_w_data_a(0 to N_CORES - 1);
signal mem_w_enable_s : mem_w_enable_a(0 to N_CORES - 1);
signal mem_w_out_s : mem_w_out_a(0 to N_CORES - 1);
signal cpu_out_s : cpu_out_a(0 to N_CORES - 1);
--signal cpu_out_s : std_logic_vector(N_CORES - 1 downto 0);
--signal cpu_stop_s : cpu_stop_a(0 to N_CORES - 1);
signal cpu_stop_s : std_logic_vector(N_CORES - 1 downto 0);
-- register used for oneshot output from each core
signal cpu_stop_s_w : std_logic_vector(N_CORES - 1 downto 0);

signal conf_reg_s : std_logic_vector(CONF_BUS - 1 downto 0);
begin
conf_reg_s <= conf_reg_i;
mem_w_address_s <= mem_w_address_array;
mem_w_data_s <= mem_w_data_array;
mem_w_enable_s <= mem_w_enable_array;
mem_w_out_array <= mem_w_out_s;
--cpu_out_o <= cpu_out_s;
gen_cores : for i in 0 to N_CORES - 1 generate
	core_rst_s(i) <= core_rst;
	cpu_cores : cpu
		port map(clk => clk,
					rst => core_rst_s(i),
					mem_w_address_i => mem_w_address_s(i),
					mem_w_data_i => mem_w_data_s(i),
					mem_w_enable_i => mem_w_enable_s(i),
					mem_w_out_o => mem_w_out_s(i),
					cpu_out => cpu_out_s(i),
					cpu_stop => cpu_stop_s(i),
					conf_reg => conf_reg_s
					);
	end generate gen_cores;
gen_regout : for i in 0 to N_CORES - 1 generate
		-- connect odd numbered signals
		conf_reg_o(i*2) <= conf_reg_i(i*2) when rising_edge(clk);
		-- even numbered signals from the cpu cores
		--conf_reg_o(i*2) <= cpu_out_s(i)(0) when (rising_edge(clk) and(cpu_stop_s_w(i) = '0')) ;
		
		-- oneshot register to enable registering the output of a core only one time
		-- drive core_rst low for the respective core to rst the cpu and the oneshot register
		oneshot: process(clk, core_rst_s, cpu_stop_s_w, cpu_stop_s)
		begin
			if core_rst_s(i) = '0' then
				cpu_stop_s_w(i) <= '0';
				
			elsif rising_edge(clk) and (cpu_stop_s_w(i) = '0') and (cpu_stop_s(i) = '1')then
				cpu_stop_s_w(i) <= '1';
				conf_reg_o(i*2+1) <= cpu_out_s(i)(0);
			end if;
		end process;
	end generate gen_regout;
-- set done flag to indicate that conf_reg_o holds the new configuration
done_o <= '1' when (cpu_stop_s_w = (cpu_stop_s_w'range => '1')) else '0';

-- register the unused configuration bits, not sure if this breaks the code if N_CORES = 64
-- in this case just remove this line
conf_reg_o(CONF_BUS - 1 downto N_CORES * 2) <= conf_reg_i (CONF_BUS - 1 downto N_CORES * 2) when rising_edge(clk);

end ca_cpu_arch;