----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:13:56 02/20/2012 
-- Design Name: 
-- Module Name:    cmd_control - cmd_control_arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use work.emb.all;
use work.cpu_package.all;

entity cmd_control is
	port (
			clk : in std_logic;
			rst : in std_logic;	

        -- Pin controller interface
		  -- for some reason this uses active low
		  -- enable signals
			pcfg_we : out std_logic;
			configdata : out std_logic_vector(MAT_PINS * 2 -1 downto 0);
			pinvalues : in std_logic_vector(MAT_PINS * 2 - 1 downto 0);

			-- ca_cpu signals
			core_rst : out std_logic;
			-- old state, this should be registered
			conf_reg_i : out std_logic_vector(CONF_BUS - 1 downto 0);
			-- new state, registered output
			conf_reg_o : in std_logic_vector(CONF_BUS - 1 downto 0);
			-- done status flag, means that all cores are finished
			done_o : in std_logic;
			
			-- memory signals
			-- config memory
			config_mem_we : out std_logic;
			config_mem_addr : out std_logic_vector(MEM_SUBADDR_BITS - 1 downto 0);
			config_mem_din : out std_logic_vector(MEM_DATA_BITS - 1 downto 0);
			config_mem_do : in std_logic_vector(MEM_DATA_BITS - 1 downto 0);

			-- ca state memory
			ca_mem_we : out std_logic;
			ca_mem_addr : out std_logic_vector(MEM_CAADDR_BITS - 1 downto 0);
			ca_mem_data_i : out std_logic_vector(MEM_DATA_BITS - 1 downto 0);
			ca_mem_data_o : in std_logic_vector(MEM_DATA_BITS - 1 downto 0)
		);
end cmd_control;

architecture cmd_control_arch of cmd_control is
type states is (idle, read_cmd_reg, dispatch_cmd, read_conf,
					read_conf_loop, read_conf_done, read_pins_loop, read_pins_done,
					conf_evo, conf_evo_loop, run_evo, run_evo_set_config, conf_evo_done,
					run_evo_write_state, run_evo_sample, run_evo_wait, conf_evo_set_config, test, read_conf_set_config);
signal state, next_state : states;
signal pin_word : std_logic_vector(2 downto 0); -- 0 to 7, 8 words.
signal pin_word_s : std_logic_vector(2 downto 0);
signal confdata_reg : std_logic_vector(MAT_PINS * 2 - 1 downto 0);
signal confdata_reg_s : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
-- used for writing parts of the sampled signal stores in confdata_reg
-- back to ca_mem
signal confdata_reg_r : std_logic_vector(MEM_DATA_BITS - 1 downto 0);

-- the two evo signals below should both or at least the sample counter
-- be configurable from the host.
signal evo_counter : std_logic_vector(7 downto 0); -- counting to 255, for 256 samples
signal evo_counter_s : std_logic_vector(7 downto 0); -- temp signal
signal evo_wait : std_logic_vector(15 downto 0); -- waitcycles until saving the sample
signal evo_wait_s : std_logic_vector(15 downto 0); -- temp signal

--test counter for measurement of frequency
signal test_cntr : std_logic_vector(15 downto 0) := (others => '0');
signal test_cntr_s : std_logic_vector(15 downto 0) := (others => '0');
begin

-- This process handles the registering when reading the configuration
-- from memory. The configuration words is saved in confdata_reg.
process(clk)
begin
	if rising_edge(clk) then
		if rst = '0' then
			--confdata_reg <= (others  => '0');
			--reset configuration to all Zs, every mode bit set to '1'
			for i in 0 to 63 loop
				confdata_reg(i*2) <= '1';
				confdata_reg(i*2+1) <= '0';
			end loop;
		end if;
		if state = read_conf or state = read_conf_loop or state = conf_evo or state = conf_evo_loop then
			case (pin_word) is
				when "000" =>
					confdata_reg(15 downto 0) <= confdata_reg_s;
					
				when "001" =>
					confdata_reg(31 downto 16) <= confdata_reg_s;
					
				when "010" =>
					confdata_reg(47 downto 32) <= confdata_reg_s;
					
				when "011" =>
					confdata_reg(63 downto 48) <= confdata_reg_s;
					
				when "100" =>
					confdata_reg(79 downto 64) <= confdata_reg_s;
					
				when "101" =>
					confdata_reg(95 downto 80) <= confdata_reg_s;
					
				when "110" =>
					confdata_reg(111 downto 96) <= confdata_reg_s;
					
				when "111" =>
					confdata_reg(127 downto 112) <= confdata_reg_s;
				when others =>
				
			end case;
		elsif state = run_evo_sample then
			confdata_reg <= pinvalues;
		end if;
	end if;
end process;

-- evo counters, sample wait delay and sample counter.
process(clk)
begin
	if rising_edge(clk) then
		if rst = '0' then
			evo_counter <= (others => '0');
			evo_wait <= (others => '0');
		elsif state = run_evo_wait then
			evo_wait <= evo_wait_s;
		elsif state = run_evo_write_state then
			evo_counter <= evo_counter_s;
		end if;
	end if;
end process;

-- counter for reading configuration from memory and writing result back
process(clk)
begin
	if rising_edge(clk) then
		if rst = '0' or (state = read_conf_done) or (state = read_pins_done) then
			pin_word <= (others => '0');
		elsif (state = read_conf) or (state = read_conf_loop) or
				(state = read_pins_loop) or state = conf_evo or
				state = conf_evo_loop or state = run_evo_write_state then
			pin_word <= pin_word_s;
		end if;
	end if;
end process;

-- process for muxing the right part of confdata_reg
-- to confdata_reg_r used for writing back right word
-- to ca state memory after sampling.
process(confdata_reg, pin_word)
begin
	case (pin_word) is
		when "000" =>
			confdata_reg_r <= confdata_reg(15 downto 0);

		when "001" =>
			confdata_reg_r <= confdata_reg(31 downto 16);
			
		when "010" =>
			confdata_reg_r <= confdata_reg(47 downto 32);
			
		when "011" =>
			confdata_reg_r <= confdata_reg(63 downto 48);
				
		when "100" =>
			confdata_reg_r <= confdata_reg(79 downto 64);
					
		when "101" =>
			confdata_reg_r <= confdata_reg(95 downto 80);
					
		when "110" =>
			confdata_reg_r <= confdata_reg(111 downto 96);
					
		when "111" =>
			confdata_reg_r <= confdata_reg(127 downto 112);
		when others =>
				
	end case; 
end process;

-- next state register
process(clk)
begin
	if rising_edge(clk) then
		if rst = '0' then
			state <= idle;
		else
			state <= next_state;
		end if;
	end if;
end process;

process(clk, rst, state)
begin
	if rising_edge(clk) then
		if rst = '0' then
			test_cntr <= (others => '0');
		else
			if state = idle then
				test_cntr <= test_cntr_s;
			end if;
		end if;
	end if;
end process;
-- next state and output logic
process(state, pinvalues, conf_reg_o, done_o, config_mem_do,
			ca_mem_data_o, pin_word, confdata_reg, evo_wait, confdata_reg_r, evo_counter, pin_word_s, test_cntr, test_cntr_s)
begin
	-- defaults
	test_cntr_s <= (others => '0');
	next_state <= state;
	confdata_reg_s <= (others => '0');
	pin_word_s <= (others => '0');
	evo_wait_s <= (others => '0');
	evo_counter_s <= (others => '0');
	-- pin control
	pcfg_we <= '0';
	configdata <= (others => '1');

	-- ca_cpu
	core_rst <= '0';
	conf_reg_i <= (others => '0');

	-- config_mem
	config_mem_we <= '0';
	config_mem_addr <= (others => '0');
	config_mem_din <= (others => '0');

	-- ca state mem
	ca_mem_we <= '0';
	ca_mem_addr <= (others => '0');
	ca_mem_data_i <= (others => '0');

	case (state) is
		when idle =>
			next_state <= read_cmd_reg;
			config_mem_we <= '1';
			config_mem_addr <= TEST_FREQ;
			test_cntr_s <= std_logic_vector(unsigned(test_cntr) + 1);
			config_mem_din <= test_cntr_s;
			
		when read_cmd_reg =>
			config_mem_addr <= CMD_REG;
			next_state <= dispatch_cmd;
		
		when dispatch_cmd =>
			case (config_mem_do) is
				when CMD_NOOP =>
					next_state <= idle;

				when CMD_CONF_PINS =>
					-- reads configuration from mem and sets the pins
					next_state <= read_conf;

				when CMD_READ_PINS =>
					-- writes the pinvalues back to memory
					next_state <= read_pins_loop;

				when CMD_CONF_EVO =>
					next_state <= conf_evo;
				
				when CMD_RUN_EVO =>
					next_state <= run_evo;
				when "1000000000000000" =>
					next_state <= test;
				when others =>
					next_state <= idle;
			end case;

		when test =>
			next_state <= idle;
			config_mem_we <= '1';
			config_mem_addr <= CMD_REG;
			config_mem_din <= CMD_NOOP;

		-- read first byte from initial config memory into a register
		when read_conf =>
			next_state <= read_conf_loop;
			config_mem_addr <= std_logic_vector(
							unsigned(CONF_WORDS_BASE) +
							unsigned(pin_word));

		-- read rest of the bytes from initial config memory into a register
		when read_conf_loop =>
			-- first word is ready to be written
			if pin_word = "111" then
				-- end loop after 8 words have
				-- been read from memory and applied to the pins
				next_state <= read_conf_set_config;
			end if;
			-- increment word counter, pin_word_s will be registered on pin_word on next state
			-- reason for the pin_word_s and pin_word split is that pin_word_s is used for reading
			-- which takes one cycle, while pin_word the address of the last read word.
			pin_word_s <= std_logic_vector(unsigned(pin_word) + 1);
			config_mem_addr <= std_logic_vector(
										unsigned(CONF_WORDS_BASE) +
										unsigned(pin_word_s));
			-- confdata_reg_s is routed to right place in the confdata_reg register based on value of pin_word.
			confdata_reg_s <= config_mem_do;

		-- write config to pin controller
		when read_conf_set_config =>
			configdata <= confdata_reg;
			pcfg_we <= '1';
			next_state <= read_conf_done;


		-- read pin values from the pin controller and write them to ca memory.
		when read_pins_loop =>
			if pin_word = "111" then
				next_state <= read_pins_done;
			end if;
			-- pin_word_s only used to increment pin_word here.
			pin_word_s <= std_logic_vector(unsigned(pin_word) + 1);
			for i in 0 to 7 loop
				if i = unsigned(pin_word) then
					ca_mem_addr <= "00000000"&pin_word;
					ca_mem_data_i <= pinvalues(15 + (i * 16) downto 16*i);
				end if;
			end loop;
			ca_mem_we <= '1';


		when conf_evo =>
			next_state <= conf_evo_loop;
			-- first config word is ready in next state
			config_mem_addr <= std_logic_vector(
							unsigned(CONF_WORDS_BASE) +
							unsigned(pin_word));

			
		when conf_evo_loop =>
			if pin_word = "111" then
				next_state <= conf_evo_set_config;
			end if;
			pin_word_s <= std_logic_vector(unsigned(pin_word) + 1);
			config_mem_addr <= std_logic_vector(
							unsigned(CONF_WORDS_BASE) +
							unsigned(pin_word_s));
			confdata_reg_s <= config_mem_do;
		

		-- ugly hack have to set and read back the first configuration
		-- else the data run through the ca cpu will miss the potential
		-- input values from the material
		when conf_evo_set_config =>
			configdata <= confdata_reg;
			pcfg_we <= '1';
			next_state <= run_evo_wait;


		when run_evo =>
			-- confdata_reg should now contain the CA initial state.
			conf_reg_i <= confdata_reg;
			core_rst <= '1';
			config_mem_we <= '1';
			config_mem_addr <= CMD_REG;
			config_mem_din <= "0000000011111111"; -- just a test value written to the CMD_REG easy to see if the CA CPU doesn't finish or if using a long time
			if done_o = '1' then
				next_state <= run_evo_set_config;
			end if;
			
		when run_evo_set_config =>
			core_rst <= '1';
			configdata <= conf_reg_o;
			pcfg_we <= '1';
			next_state <= run_evo_wait;
		
		when run_evo_wait =>
		-- this is just so the signal has time going through the pin controller and
		-- the material and back through the synchronizers. Should at least be 3 cycles.
			evo_wait_s <= std_logic_vector(unsigned(evo_wait) + 1);
			if evo_wait = "1111111111111111" then -- 7th cycle
				next_state <= run_evo_sample;
			end if;
		
		when run_evo_sample =>
			-- the pinvalues is registered in this state, see the process above
			-- handling the confdata_reg register.
			next_state <= run_evo_write_state;
		
		when run_evo_write_state =>
			-- confdata_reg now contains applied signals and response from material
			-- write it to ca_mem
			evo_counter_s <= evo_counter;
			pin_word_s <= std_logic_vector(unsigned(pin_word) + 1);
			ca_mem_we <= '1';
			--ca_mem_addr <= "00000000"&pin_word;
			ca_mem_addr <= evo_counter&pin_word;
			ca_mem_data_i <= confdata_reg_r;
			
			if pin_word = "111" then
				-- done writing sample words, check if there are more samples
				if evo_counter = "11111111" then -- max(256) samples for now
					next_state <= conf_evo_done;
					evo_counter_s <= (others => '0');
				else
					-- more samples
					evo_counter_s <= std_logic_vector(unsigned(evo_counter) + 1);
					next_state <= run_evo;
				end if;
			end if;
			
		when conf_evo_done =>
			next_state <= idle;
			config_mem_we <= '1';
			config_mem_addr <= CMD_REG;
			config_mem_din <= CMD_NOOP;

		when read_pins_done =>
			next_state <= idle;
			config_mem_we <= '1';
			config_mem_addr <= CMD_REG;
			config_mem_din <= CMD_NOOP;
		
		when read_conf_done =>
			next_state <= idle;
			config_mem_we <= '1';
			config_mem_addr <= CMD_REG;
			config_mem_din <= CMD_NOOP;

	end case;
end process;
end cmd_control_arch;

