library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.cpu_package.all;

entity decode is
	port	(
		clk : in std_logic;
		rst : in std_logic;
		instr_i : in std_logic_vector(IDATA_BUS - 1 downto 0);
		-- data from writeback
		wb_out_i : in std_logic_vector(DDATA_BUS - 1 downto 0);
		-- control signals
		-- input
		ex_opc_i : in std_logic_vector(OPC_BUS - 1 downto 0);
		wb_opc_i : in std_logic_vector(OPC_BUS - 1 downto 0);
		-- output
		id_alu_a_fw_o : out std_logic;
		id_push_o : out std_logic;
		id_stop_o : out std_logic;
		id_write_out_o : out std_logic;
		id_opc_o : out std_logic_vector(OPC_BUS - 1 downto 0);

		id_reg_0_o : out std_logic_vector(DDATA_BUS - 1 downto 0);
		id_reg_1_o : out std_logic_vector(DDATA_BUS - 1 downto 0);
		id_alu_func_o : out std_logic_vector(FUNC_BUS - 1 downto 0)
		);
end decode;

architecture decode_arch of decode is
signal regfile : std_logic_vector(REGFILE_SIZE - 1 downto 0);
signal id_alu_func_s : std_logic_vector(FUNC_BUS - 1 downto 0);
-- unary argument/1st argument
--signal id_reg_0_s : std_logic_vector(DDATA_BUS - 1 downto 0);
-- binary 2nd argument
--signal id_reg_1_s : std_logic_vector(DDATA_BUS - 1 downto 0);
signal id_alu_a_fw_s : std_logic;
signal id_push_s : std_logic;
signal id_stop_s : std_logic;
signal id_write_out_s : std_logic;
signal id_opc_s : std_logic_vector(OPC_BUS - 1 downto 0);
-- control signal for forwarding mux
signal id_fw_s : std_logic;
-- output from id_fw_s mux
signal id_fw_out_s : std_logic_vector(DDATA_BUS - 1 downto 0);
-- stack control signals
signal stack_func_s : std_logic_vector(1 downto 0);
begin	
id_opc_s <= instr_i(IFMT_OPC'range);
id_alu_func_s <= instr_i(IFMT_FUNC'range);
control : process(clk, rst, instr_i, id_opc_s, ex_opc_i, wb_opc_i)
	begin
		id_alu_a_fw_s <= '0';
		id_push_s <= '0';
		id_stop_s <= '0';
		id_write_out_s <= '0';
		id_fw_s <= '0';
		stack_func_s <= STACK_NOP;
		case (id_opc_s & ex_opc_i & wb_opc_i) is
			-- opc_in
			when OPC_IN & OPC_IN & OPC_IN =>
				id_push_s <= '1';
				stack_func_s <= STACK_PUSH;
			when OPC_IN & OPC_IN & OPC_OUT =>
				id_push_s <= '1';
			when OPC_IN & OPC_OUT & OPC_IN =>
				id_push_s <= '1';

			when OPC_IN & OPC_OUT & OPC_OUT =>
				id_push_s <= '1';

			-- opc_out
			when OPC_OUT & OPC_IN & OPC_IN =>
				id_alu_a_fw_s <= '1';
				stack_func_s <= STACK_PUSH;
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_OUT & OPC_IN & OPC_OUT =>
				id_alu_a_fw_s <= '1';
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_OUT & OPC_OUT & OPC_IN =>
				stack_func_s <= STACK_POP1;
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_OUT & OPC_OUT & OPC_OUT =>
				stack_func_s <= STACK_POP1;
				id_stop_s <= '1';
				id_write_out_s <= '1';

			-- opc_in/opc_alu
			when OPC_IN & OPC_IN & OPC_ALU =>
				id_push_s <= '1';
				stack_func_s <= STACK_PUSH;
			when OPC_IN & OPC_ALU & OPC_IN =>
				id_push_s <= '1';

			when OPC_IN & OPC_ALU & OPC_ALU =>
				id_push_s <= '1';

			when OPC_ALU & OPC_IN & OPC_IN =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					id_fw_s <= '1';
				else
					stack_func_s <= STACK_PUSH;
				end if;

			when OPC_ALU & OPC_IN & OPC_ALU =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					id_fw_s <= '1';
				else
					stack_func_s <= STACK_PUSH;
				end if;

			when OPC_ALU & OPC_ALU & OPC_IN =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP1;
				end if;

			when OPC_ALU & OPC_ALU & OPC_ALU =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP1;
				end if;

			-- opc_out/opc_alu
			when OPC_OUT & OPC_OUT & OPC_ALU =>
				stack_func_s <= STACK_POP1;
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_OUT & OPC_ALU & OPC_OUT =>
				id_alu_a_fw_s <= '1';
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_OUT & OPC_ALU & OPC_ALU =>
				id_alu_a_fw_s <= '1';
				id_stop_s <= '1';
				id_write_out_s <= '1';

			when OPC_ALU & OPC_OUT & OPC_OUT =>
				id_push_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP2;
				else
					stack_func_s <= STACK_POP1;
				end if;

			when OPC_ALU & OPC_OUT & OPC_ALU =>
				id_push_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP2;
				else
					stack_func_s <= STACK_POP1;
				end if;

			when OPC_ALU & OPC_ALU & OPC_OUT =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP1;
				end if;

			-- opc_out/opc_alu/opc_in
			when OPC_IN & OPC_OUT & OPC_ALU =>
				id_push_s <= '1';

			when OPC_IN & OPC_ALU & OPC_OUT =>
				id_push_s <= '1';

			when OPC_ALU & OPC_IN & OPC_OUT =>
				id_push_s <= '1';
				id_alu_a_fw_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP1;
				end if;

			when OPC_ALU & OPC_OUT & OPC_IN =>
				id_push_s <= '1';
				if instr_i(BINARY_OP) = '1' then
					stack_func_s <= STACK_POP2;
				else
					stack_func_s <= STACK_POP1;
				end if;

			when OPC_OUT & OPC_IN & OPC_ALU =>
				id_alu_a_fw_s <= '1';
				id_stop_s <= '1';
				id_write_out_s <= '1';
				stack_func_s <= STACK_PUSH;

			when OPC_OUT & OPC_ALU & OPC_IN =>
				id_alu_a_fw_s <= '1';
				id_stop_s <= '1';
				id_write_out_s <= '1';
			when others =>
		end case;
	end process;
-- muxing wb_out_i and regfile(0), forwarding past the stack
id_fw_out_s(0) <= wb_out_i(0) when id_fw_s = '1' else regfile(0);

output_reg : process(clk, rst)
	begin
		if rst = '0' then
			id_reg_0_o <= (others => '0');
			id_reg_1_o <= (others => '0');
			id_alu_a_fw_o <= '0';
			id_push_o <= '0';
			id_stop_o <= '0';
			id_write_out_o <= '0';
			id_opc_o <= (others => '0');
			id_alu_func_o <= (others => '0');

		elsif rising_edge(clk) then
			id_reg_0_o <= id_fw_out_s;
			id_reg_1_o(0) <= regfile(0);
			id_alu_a_fw_o <= id_alu_a_fw_s;
			id_push_o <= id_push_s;
			id_stop_o <= id_stop_s;
			id_write_out_o <= id_write_out_s;
			id_opc_o <= id_opc_s;
			id_alu_func_o <= id_alu_func_s;
		end if;
	end process;

stack : process(clk, rst)
	begin
		if rst = '0' then
			regfile <= (others => '0');
		elsif rising_edge(clk) then
		case stack_func_s is
			when STACK_NOP => --nothing

			when STACK_PUSH => -- push
				regfile <= regfile(regfile'left - 1 downto regfile'right) & wb_out_i;
			when STACK_POP1 => -- pop unary
				regfile <= regfile(0)&regfile(regfile'left downto regfile'right + 1);
			when STACK_POP2 => -- pop binary
				regfile <= regfile(0)&regfile(1)&regfile(regfile'left downto regfile'right + 2);
			when others =>
		end case;
		end if;
	end process;
end decode_arch;
