library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.cpu_package.all;

entity execute is
	port	(
		clk : in std_logic;
		rst : in std_logic;
		conf_reg : in std_logic_vector(CONF_BUS - 1 downto 0);
		-- instruction signals
		id_reg_0 : in std_logic_vector(DDATA_BUS - 1 downto 0);
		id_reg_1 : in std_logic_vector(DDATA_BUS - 1 downto 0);
		id_alu_func : in std_logic_vector(FUNC_BUS - 1 downto 0);
		-- controll signals
		id_alu_a_fw : in std_logic;
		id_opc : in std_logic_vector(OPC_BUS - 1 downto 0);
		id_push : in std_logic;
		id_stop : in std_logic;
		id_write_out : in std_logic;

		-- execute output registers
		ex_alu_out : out std_logic_vector(DDATA_BUS - 1 downto 0);
		ex_r_out : out std_logic;
		ex_w_out : out std_logic;
		ex_push : out std_logic;
		ex_stop : out std_logic;
		ex_opc : out std_logic_vector(OPC_BUS - 1 downto 0);

		-- from writeback
		wb_out_i : in std_logic_vector(DDATA_BUS - 1 downto 0)
		);
end execute;

architecture execute_arch of execute is
-- alu inputs
signal alu_in_a : std_logic_vector(DDATA_BUS - 1 downto 0);
signal alu_in_b : std_logic_vector(DDATA_BUS - 1 downto 0);
--signal alu_func : std_logic_vector(FUNC_BUS - 1 downto 0);
-- alu outputs
signal alu_out : std_logic_vector(DDATA_BUS - 1 downto 0);
signal alu_status : std_logic_vector(STATUS_BUS - 1 downto 0);

-- execute out registers
signal ex_out_s : std_logic_vector(DDATA_BUS - 1 downto 0);
signal ex_r_out_s : std_logic;
signal ex_w_out_s : std_logic;
begin
u_alu : alu
	port map(alu_in_a => alu_in_a,
		alu_in_b => alu_in_b,
		alu_func => id_alu_func,
		alu_out => alu_out,
		alu_status => alu_status);
u_io_reg : io_reg
	port map(clk => clk,
		rst => rst,
		conf_reg => conf_reg,
		conf_addr => id_alu_func,
		w_enable => id_write_out,
		data_in => alu_in_a,
		w_out => ex_w_out_s,
		r_out => ex_r_out_s);
ex_w_out <= ex_w_out_s;
--ex_r_out <= ex_r_out_s;

-- forwarding logic
alu_in_a <= wb_out_i when id_alu_a_fw = '1' else
		id_reg_0;
-- binary alu instruction in execute and second operand
-- got forwarded past the stack, since it is on top
-- of the stack we have to switch it top alu_in_b
alu_in_b <= id_reg_0 when ((id_alu_a_fw = '1') and (id_alu_func(BINARY_OP) = '1')) else id_reg_1;

exec_regs : process (clk, rst)
begin
		if rst = '0' then
			ex_alu_out <= (others => '0');
			ex_stop <= '0';
			ex_push <= '0';
			ex_opc <= (others => '0');
			ex_r_out <= '0';
		elsif rising_edge(clk) then
			ex_alu_out <= alu_out;
			ex_stop <= id_stop;
			ex_push <= id_push;
			ex_opc <= id_opc;
			ex_r_out <= ex_r_out_s;
		end if;
	end process;
end execute_arch;
