library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.cpu_package.all;

entity mem is
	generic	(
		MEM_ADDR_BUS : integer := IADDR_BUS;
		MEM_DATA_BUS : integer := IDATA_BUS * 2
		);
	port	(
		-- read port
		r_address : in std_logic_vector(MEM_ADDR_BUS downto 0); -- + 1
		r_out : out std_logic_vector(MEM_DATA_BUS/2 - 1 downto 0);

		-- write/read port
		w_address : in std_logic_vector(MEM_ADDR_BUS - 1 downto 0);
		w_data : in std_logic_vector(MEM_DATA_BUS - 1 downto 0);
		w_enable : in std_logic;
		w_out : out std_logic_vector(MEM_DATA_BUS - 1 downto 0);

		clk : in std_logic
		);
end mem;

architecture mem_arch of mem is
	constant MEM_SIZE : integer := 2 ** MEM_ADDR_BUS;
--	type mem_type is array (0 to MEM_SIZE - 1)
	type mem_type is array (MEM_SIZE -1 downto 0)
		of std_logic_vector(MEM_DATA_BUS - 1 downto 0);
	signal mem_storage : mem_type := (others => (others => '0'));
	--signal r_address_reg : std_logic_vector(MEM_ADDR_BUS downto 0);
	--signal w_address_reg : std_logic_vector(MEM_ADDR_BUS - 1 downto 0);
begin
	process(clk, mem_storage,r_address, w_address)
	begin
		if rising_edge(clk) then
			if w_enable = '1' then
				mem_storage(to_integer(unsigned(w_address))) <= w_data;
			end if;
			--r_address_reg <= r_address;
			--w_address_reg <= w_address;
						end if;
		if r_address(0) = '0' then
			r_out <= mem_storage(to_integer(unsigned(r_address(MEM_ADDR_BUS downto 1))))(MEM_DATA_BUS/2 - 1 downto 0);
		else
			r_out <= mem_storage(to_integer(unsigned(r_address(MEM_ADDR_BUS downto 1))))(MEM_DATA_BUS - 1 downto MEM_DATA_BUS/2);
		end if;
		w_out <= mem_storage(to_integer(unsigned(w_address)));

	end process;
end mem_arch;
