library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.all;
use work.emb.all;

entity pin_config_control is
	generic ( npins : positive := 64 );

	port (	clk         : in std_logic;
				rst         : in std_logic;
				write       : in std_logic;
				configdata  : in std_logic_vector((2 * npins - 1) downto 0);
				pin_io      : inout std_logic_vector((npins - 1) downto 0);
				pin_val     : out std_logic_vector((2 * npins - 1) downto 0) );
end pin_config_control;

architecture pin_config_control_arch of pin_config_control is
	signal config_reg : std_logic_vector( ((2 * npins) - 1) downto 0);
	signal pinvalue : std_logic_vector(npins - 1 downto 0);
begin
		 
	pin_setup : for pin_index in 0 to npins - 1 generate
	begin
		pin : entity work.pin
		port map (
						clk => clk,
						cfgbits => config_reg((2*pin_index + 1) downto (2 * pin_index)),
						oval    => pinvalue(pin_index),
						pin_out => pin_io(pin_index) );
		end generate pin_setup;
   
    --Capture pin configuration data if write enabled.
	write_config : process(clk) is
	begin
		if (rising_edge(clk) ) then 
			if (rst = '0') then
				-- reset this to all 1s so it doesnt drive the tristate from the FPGA at start
				config_reg <= (others => '1');
			elsif (write = '1') then
				config_reg <= configdata;
			end if;
		end if;
	end process;
	
new_pin_values : for pin in 0 to npins - 1 generate
begin
	-- even numbered is mode bits
	pin_val(pin * 2) <= config_reg(pin * 2);
	-- odd numbered is value bits, which comes from the materials synchronizers.
	pin_val(pin * 2 + 1) <= pinvalue(pin);
end generate new_pin_values;
end pin_config_control_arch;

