library ieee;

use ieee.std_logic_1164.all;
use work.all;
use work.emb.all;
use work.cpu_package.all;


entity toplevel is 
    port (  
            main_clk : in std_logic;
            main_rst : in std_logic;
            -- Memory bus and control signals 
            mem_addr  : in std_logic_vector(MC_ADDR_BITS - 1 downto 0);
            mem_data  : inout std_logic_vector(MC_DATA_BITS  - 1 downto 0);
            mem_cs    : in std_logic;
            mem_re    : in std_logic;
            mem_we    : in std_logic;
            mem_wait  : out std_logic;

            io_port : inout std_logic_vector(63 downto 0)
            );
end entity;

architecture rtl of toplevel is

 --   signal usr_module_data_i  : std_logic_vector(USR_DATA_BITS - 1 downto 0);
 --   signal usr_module_data_o  : std_logic_vector(USR_DATA_BITS - 1 downto 0);
 --   signal usr_module_addr  : std_logic_vector(USR_ADDR_BITS - 1 downto 0);
 --   signal usr_module_re    : std_logic;
 --   signal usr_module_we    : std_logic;
 --   signal usr_module_cs    : std_logic;
 --   signal usr_module_wait  : std_logic;
    signal pincfg_we : std_logic;
	 signal pincfg_data : std_logic_vector(MAT_PINS * 2 -1 downto 0);
	 signal pincfg_val: std_logic_vector(MAT_PINS * 2 -1 downto 0);
   -- signal usr_module_pincfg_oe : std_logic;
   --- signal usr_module_pincfg_data : std_logic_vector(15 downto 0);
   -- signal usr_module_pincfg_bankaddr : bank_sel_type;
    
 --   signal pinctrl_pinvalues : std_logic_vector(7 downto 0);

  --  signal memctl_we : std_logic;
 --   signal memctl_addr : std_logic_vector(USR_ADDR_BITS - 1 downto 0);
  --  signal memctl_din : std_logic_vector(USR_DATA_BITS - 1 downto 0);
--    signal memctl_do : std_logic_vector(USR_DATA_BITS - 1 downto 0);

    -- synchronized signals
    signal syn_mem_cs : std_logic;    
    signal syn_mem_re : std_logic;    
    signal syn_mem_we : std_logic;

		-- ca cpu signals
	signal mem_w_address_array : mem_w_address_a(0 to N_CORES - 1);
	signal mem_w_data_array : mem_w_data_a(0 to N_CORES - 1);
	signal mem_w_enable_array : mem_w_enable_a(0 to N_CORES - 1);
	signal mem_w_out_array : mem_w_out_a(0 to N_CORES - 1);
	
	
	-- signals between cmd_control and other modules
	-- ca_cpu
	signal core_rst : std_logic;
			-- old state, this should be registered
	signal conf_reg_i : std_logic_vector(CONF_BUS - 1 downto 0);
			-- new state, registered output
	signal conf_reg_o : std_logic_vector(CONF_BUS - 1 downto 0);
			-- done status flag, means that all cores are finished
	signal done_o : std_logic;

	-- config_mem
	signal a_config_mem_we : std_logic;
	signal a_config_mem_addr : std_logic_vector(MEM_SUBADDR_BITS - 1 downto 0);
	signal a_config_mem_din : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	signal a_config_mem_do : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	
	-- ca_mem
	signal ca_mem_we : std_logic;
	signal ca_mem_addr : std_logic_vector(MEM_CAADDR_BITS - 1 downto 0);
	signal ca_mem_data_i : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	signal ca_mem_data_o : std_logic_vector(MEM_DATA_BITS - 1 downto 0);


	-- signals between uc_interface and memories
	-- config_mem
	signal b_config_mem_we  : std_logic;
	signal b_config_mem_addr: std_logic_vector(MEM_SUBADDR_BITS - 1 downto 0);
	--this goes in to this unit, from internal memory
	signal b_config_mem_do : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	--this goes out of this unit, into internal memory
	signal b_config_mem_din  : std_logic_vector(MEM_DATA_BITS - 1 downto 0);

	-- ca state mem
	-- interface to CA state memory
	signal ca_state_we : std_logic;
	signal ca_state_addr : std_logic_vector(MEM_CAADDR_BITS - 1 downto 0);
	--signal ca_state_data_i : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	signal ca_state_data_o : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
begin
    
 --   memory_controller : entity work.memory_controller
 --   port map (
 --       clk => main_clk,
 --       rst => main_rst,
		  -- signals from the uc3
 --       e_addr => mem_addr,
 --       e_data => mem_data,
 --       e_cs => syn_mem_cs,
 --       e_re => syn_mem_re,
 --       e_we => syn_mem_we, 
 --       e_wait => mem_wait,
		  -- signals to the usr module
 ---       i_addr => usr_module_addr,
 --       i_data_i => usr_module_data_o,
 --       i_data_o => usr_module_data_i,
 --       i_re => usr_module_re,
 --       i_we => usr_module_we,
 --       i_cs => usr_module_cs,
 --       i_wait => usr_module_wait,
		  -- signals to the memory module
 --       mem_we => memctl_we,
 --       mem_addr => memctl_addr,
 --       mem_din => memctl_din,
 --       mem_do => memctl_do,
		  -- instruction memory signals
--		  mem_w_address_array => mem_w_address_array,
	--	  mem_w_data_array => mem_w_data_array,
--		  mem_w_enable_array => mem_w_enable_array,
	--	  mem_w_out_array => mem_w_out_array
  --      );

	ca_cpu : entity work.ca_cpu
	port map (
			clk => main_clk,
			rst => main_rst,
			-- hold reset high until you want to start the cores
			core_rst => core_rst,
			-- memory
			mem_w_address_array => mem_w_address_array,
			mem_w_data_array => mem_w_data_array,
			mem_w_enable_array => mem_w_enable_array,
			mem_w_out_array => mem_w_out_array,
			-- old state, this should be registered
			conf_reg_i => conf_reg_i,
			-- new state, registered output
			conf_reg_o => conf_reg_o,
			-- done status flag, means that all cores are finished
			done_o => done_o
			);

	uc_interface : entity work.uc_interface
	port map (
			clk => main_clk,
			rst => main_rst,

			uc_addr => mem_addr,
			uc_data => mem_data,
			uc_cs => syn_mem_cs,
			uc_re => syn_mem_re,
			uc_we => syn_mem_we,
			uc_wait => mem_wait,
			-- config memory, config_mem
			mem_we => a_config_mem_we,
			mem_addr => a_config_mem_addr,
			mem_do => a_config_mem_do,
			mem_din => a_config_mem_din,
			-- write port of instruction memory inside ca cpu module
			ca_inst_addr => mem_w_address_array,
			ca_inst_data_i => mem_w_data_array,
			ca_inst_we => mem_w_enable_array,
			ca_inst_data_o => mem_w_out_array,
			-- read port of ca state memory, ca_mem
			ca_state_addr => ca_state_addr,
			--ca_state_data_i => ca_state_data_i,
			ca_state_we => ca_state_we,
			ca_state_data_o => ca_state_data_o
			);

	ca_mem : entity work.ca_mem
	port map (
			clk => main_clk,
			--write/read port
			a_we => ca_mem_we,
			a_addr => ca_mem_addr,
			a_data_i => ca_mem_data_i,
			a_data_o => ca_mem_data_o,
			--read port
			b_addr => ca_state_addr,
			--b_data_i => ca_state_data_i,
			b_data_o => ca_state_data_o
			);
 
	config_mem : entity work.config_mem
	port map (
			clk => main_clk,
			a_we => a_config_mem_we,
			a_addr => a_config_mem_addr,
			a_din => a_config_mem_din,
			a_do => a_config_mem_do,
			b_we => b_config_mem_we,
			b_addr => b_config_mem_addr,
			b_din => b_config_mem_din,
			b_do => b_config_mem_do
			);
		cmd_control : entity work.cmd_control
		port map (
				clk => main_clk,
				rst => main_rst,
				-- config_mem
				config_mem_we => b_config_mem_we,
				config_mem_addr => b_config_mem_addr,
				config_mem_din => b_config_mem_din,
				config_mem_do => b_config_mem_do,
				-- ca_cpu
				core_rst => core_rst,
				conf_reg_i => conf_reg_i,
				conf_reg_o => conf_reg_o,
				done_o => done_o,
				-- ca_mem
				ca_mem_we => ca_mem_we,
				ca_mem_addr => ca_mem_addr,
				ca_mem_data_i => ca_mem_data_i,
				ca_mem_data_o => ca_mem_data_o,
				-- pin control
				pcfg_we => pincfg_we,
				configdata => pincfg_data,
				pinvalues => pincfg_val
					);
 --    usr_module : entity work.usr_module 
 --    port map (
 --        clk => main_clk,
 --         rst => main_rst,
  --        data_i => usr_module_data_i,  
   --       data_o => usr_module_data_o,  
   --       addr => usr_module_addr,
          --nwait => usr_module_wait,
          --re => usr_module_re,
          --we => usr_module_we,
--          cs => usr_module_cs,

          -- Pin controller lines
--          pcfg_we => usr_module_pincfg_we,
          --pcfg_oe => usr_module_pincfg_oe,
          --configdata => usr_module_pincfg_data,
--          bank_addr => usr_module_pincfg_bankaddr,
--          pinvalues => pinctrl_pinvalues,

				-- signals to ca cpu
--				core_rst => core_rst,
--				conf_reg_i => conf_reg_i,
--				conf_reg_o => conf_reg_o,
--				done_o => done_o,
--				conf_reg_1 => conf_reg_1
--          );
  
 --       memory : entity work.memory
   --     generic map (
  --          width => MEM_DATA_BITS,
  --          length => MEM_LENGTH,
 --           addr_bits => MEM_ADDR_BITS
--            )
--        port map (
--            clk => main_clk,
  --          we => memctl_we,
 --           a => memctl_addr,
  --          d_in => memctl_do,
  --          d_out => memctl_din
  --          );

        
        -- We need three synchronizers to clock in 
        -- the SMC control signals into the FPGA.
        cs_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_cs,
            wso => syn_mem_cs ); 

        re_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_re,
            wso => syn_mem_re ); 

        we_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_we,
            wso => syn_mem_we ); 

 --   pin_controller : entity pin_control 
 --       port map (
 --           clk => main_clk,
 --           rst => main_rst,
  --          write => usr_module_pincfg_we,
 --           read => usr_module_pincfg_oe,
 --           bank_addr => usr_module_pincfg_bankaddr,
 --           configdata => usr_module_pincfg_data,
--            pins => io_port, -- to the outside.
 --           pin_outdata => pinctrl_pinvalues );

	pin_config : entity pin_config_control
		generic map (
			npins => MAT_PINS
			)
		port map (
				clk => main_clk,
				rst => main_rst,
				write => pincfg_we,
				configdata => pincfg_data,
				pin_io => io_port,
				pin_val => pincfg_val
				);

end architecture rtl;
