----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:40:04 02/08/2012 
-- Design Name: 
-- Module Name:    uc_interface - uc_interface_arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use work.emb.all;
use work.cpu_package.all;

entity uc_interface is
    port(  
            clk     : in std_logic;
            rst     : in std_logic;

            -- Interface to the external SM controller
            -- This interface mimics a SM chip
            uc_addr  : in std_logic_vector(MEM_ADDR_BITS - 1 downto 0);
            uc_data  : inout std_logic_vector(MEM_DATA_BITS  - 1 downto 0);
            uc_cs    : in std_logic;
            uc_re    : in std_logic;
            uc_we    : in std_logic;
            uc_wait  : out std_logic;
				
            -- Interface to config memory
            mem_we  : out std_logic;
            mem_addr: out std_logic_vector(MEM_SUBADDR_BITS - 1 downto 0);
				--this goes in to this unit, from internal memory
            mem_do : in std_logic_vector(MEM_DATA_BITS - 1 downto 0);
				--this goes out of this unit, into internal memory
            mem_din  : out std_logic_vector(MEM_DATA_BITS - 1 downto 0);

				-- interface to the write/read port on the CA CPU instruction memory
				-- address to write/read to
				ca_inst_addr : out mem_w_address_a(0 to N_CORES - 1);
				-- data to write to mem
				ca_inst_data_i : out mem_w_data_a(0 to N_CORES - 1);
				-- w enable
				ca_inst_we : out mem_w_enable_a(0 to N_CORES - 1);
				-- read from mem
				ca_inst_data_o : in mem_w_out_a(0 to N_CORES - 1);
				
				-- interface to CA state memory
				ca_state_addr : out std_logic_vector(MEM_CAADDR_BITS - 1 downto 0);
				--ca_state_data_i : out std_logic_vector(MEM_DATA_BITS - 1 downto 0);
				ca_state_we : out std_logic;
				ca_state_data_o : in std_logic_vector(MEM_DATA_BITS - 1 downto 0)
			);
end uc_interface;

architecture uc_interface_arch of uc_interface is
type state_type is (idle, write_mem, read_mem, write_mem_rdy, read_mem_rdy, read_mem_hack);
signal state, next_state : state_type;

--
signal addr_s : std_logic_vector(MEM_ADDR_BITS - 1 downto 0);
signal data_i_s : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
signal we_s : std_logic;
signal data_o_s : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
signal data_o_sreg : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
--registers
signal addr_sreg : std_logic_vector(MEM_ADDR_BITS - 1 downto 0);
signal data_i_sreg : std_logic_vector(MEM_DATA_BITS - 1 downto 0);
signal we_sreg : std_logic;

--edge detection signals
signal delay_reg : std_logic;
signal pulse : std_logic;
begin
	-- registers to seperate the uc3 from the memory
	-- registers address, data inn and write enable signals
	-- between the processes so that the routing of the memory ports does
	-- not change unless the register changes, which only happens when
	-- chip select from the uc3 is low.
	process(clk, rst, state)
	begin
		if rising_edge(clk) then
			if rst= '0' then
				addr_sreg <= (others => '0');
				data_i_sreg <= (others => '0');
				--we_sreg <= '0';
				data_o_sreg <= (others =>'0');
			--elsif uc_cs = '0' then
			else
				addr_sreg <= addr_s;
				if state = write_mem then
					data_i_sreg <= data_i_s;
					--we_sreg <= we_s;
				end if;
				if state = read_mem then
					data_o_sreg <= data_o_s;
				end if;
			--elsif uc_cs ='1' then
			--	addr_sreg <= (others => '0');
			--	data_i_sreg <= (others => '0');
			--	we_sreg <= '0';
			end if;
		end if;
	end process;
	
	process(clk, rst)
	begin
		if rising_edge(clk) then
			if rst = '0' then
				state <= idle;
			else
				state <= next_state;
			end if;
		end if;
	end process;
	
	-- rising edge detection used for the write enable signal
	process(clk, uc_we, delay_reg)
	begin
		if rising_edge(clk) then
			if rst = '0' then
				delay_reg <= '0';
			else
				delay_reg <= uc_we;
			end if;
		end if;
		pulse <= (not delay_reg) and uc_we;
	end process;

	process(state, uc_addr, uc_data, uc_cs, uc_we, uc_re, data_o_s, pulse, data_o_sreg)
	begin
		--defaults
		uc_wait <= '1';
		uc_data <= (others => 'Z');
		addr_s <= uc_addr;
		we_s <= '0';
		data_i_s <= uc_data(7 downto 0)&uc_data(15 downto 8);
		next_state <= state;
		we_sreg <= '0';

		case(state) is
			when idle =>
				if uc_cs = '0' and uc_we = '0' then
					next_state <= write_mem_rdy;
					--we_s <= '1';
					uc_wait <= '0';

				elsif uc_cs = '0' and uc_re = '0' then
					next_state <= read_mem_rdy;
					uc_wait <= '0';
				end if;

			when write_mem =>
				--if rdy_s = '1' then
					-- the fpga memory is finished
			--		uc_wait <= '1';
			--	else
			--uc_wait <= '0';
			--	end if;
				--if pulse = '1' then
					we_sreg <= pulse;
				--end if;
				if uc_cs = '1' or uc_we = '1' then
					-- uc3 is done
					next_state <= idle;
				end if;
				
			when read_mem =>
				-- tell the uc3 to wait until the memory is rdy.
				-- indicating that the read is actually completed.
				--if rdy_s = '1' then
				--	uc_wait <= '1';
				--else
				--uc_wait <= '0';
				--end if;
				

				-- since the address is routed to all memories
				-- the correct out port of the right memory is
				-- muxed on in port data_o_s.
				uc_data <= data_o_sreg(7 downto 0)&data_o_sreg(15 downto 8);
				
				-- must hold the data on the bus until the uc3 acks
				-- NRD is used as controlling signal, if using NRD
				-- in this interface violates the data hold time,
				-- use NCS to control isntead since that is deasserted
				-- after NRD on the uc3.
				if uc_re = '1' or uc_cs = '1' then
					next_state <= idle;
				end if;
			when read_mem_hack =>
				uc_data <= data_o_sreg(7 downto 0)&data_o_sreg(15 downto 8);
			when write_mem_rdy =>
				next_state <= write_mem;
				uc_wait <= '0';
			when read_mem_rdy =>
				next_state <= read_mem;
				uc_wait <= '0';

			when others =>
				next_state <= idle;
		end case;
	end process;
	
-- memory interface muxing, pick right memory based on address space layout
addrspace_mux : process (addr_sreg, data_i_sreg, mem_do, we_sreg, ca_inst_data_o, ca_state_data_o )
	begin
	--defaults
	data_o_s <= (others => '0');
	-- config mem
	mem_we <= '0';
	mem_addr <= addr_sreg(9 downto 0);
	--mem_do : in std_logic_vector(MEM_DATA_BITS - 1 downto 0);
	mem_din <= data_i_sreg;
	
	--ca state mem
	ca_state_addr <= addr_sreg(10 downto 0);
	--ca_state_data_i <= data_i_sreg;
	ca_state_we <= '0';
	--ca_state_data_o : in std_logic_vector(MEM_DATA_BITS - 1 downto 0)

	for i in 0 to N_CORES-1 loop
		ca_inst_addr(i) <= addr_sreg(3 downto 0);
		ca_inst_data_i(i) <= data_i_sreg;
		ca_inst_we(i) <= '0';
	end loop;
	
		case(addr_sreg(11 downto 10)) is
			when "00" =>
				--output of memory
				data_o_s <= mem_do;
				--write signal
				mem_we <= we_sreg;

			when "01" => -- ca instruction mem
				for i in 0 to N_CORES-1 loop
					if i = to_integer(unsigned(addr_sreg(9 downto 4))) then
						ca_inst_we(i) <= we_sreg; -- write signal
						data_o_s <= ca_inst_data_o(i); -- output from mem
					end if;
				end loop;
			when "10" => -- ca state mem
				data_o_s <= ca_state_data_o; -- memory output
				ca_state_we <= we_sreg; -- write signal
			when "11" => -- other part of ca state mem
				data_o_s <= ca_state_data_o; -- memory output
				ca_state_we <= we_sreg; -- write signal
			when others =>
		end case;
	end process;
end uc_interface_arch;
