#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "ca_program.h"
/*
 * Fills a ca_program_t structure with the contents of infile.
 */
int
read_program (ca_program_t* program, char* infile)
{
	FILE* ifp;
	uint8_t* tmp_data = calloc(64, sizeof(uint8_t));
	int c;
	uint16_t len=0;

	if ( (ifp = fopen(infile, "rb")) == NULL ) {
		fprintf(stderr, "Error opening file %s for reading.", infile);
		return 1;
	}

	while ( (c = fgetc(ifp)) != EOF ) {
		tmp_data[len] = (uint8_t)c;
		len++;
	}

	program->len = len;
	program->data = tmp_data;
	return 0;
}
/*
 * Uses read_program to read in the programs for all the
 * cores and returns a pointer to a ca_program array.
 * Each file must adhere to the filename convention used
 * by sasm.py: basename-corenumber.bin, corenumber uses
 * 2 digits.
 */

ca_program_t**
read_programs (char* if_basename)
{
	ca_program_t** prg_array = malloc(32 * sizeof(ca_program_t*));
	ca_program_t *tmp_prg;
	int i;
	/*+8 for -00.bin and 0 */
	char * fname;

	for ( i = 0; i < 32; i++ ) {
		asprintf(&fname, "%s-%.2d.bin",if_basename, i);
		fprintf(stdout,"%s\n", fname);
		
		tmp_prg = malloc(sizeof(ca_program_t));
		if(read_program(tmp_prg, fname)) {
			fprintf(stderr, "Error reading file: %s\n", fname);
			exit(1);
		}
		prg_array[i] = tmp_prg;
		free(fname);
	}

	return prg_array;
}

/*
 * Reads in initial config from basename.init
 *
*/
uint8_t*
read_initial_state(char *if_basename)
{
	FILE* ifp;
	uint8_t *tmp_data = calloc(16, sizeof(uint8_t));
	int c;
	int len=0;
	char *fname;
	asprintf(&fname, "%s.init",if_basename);
	
	if ( (ifp = fopen(fname, "rb")) == NULL ) {
		fprintf(stderr, "Error opening file %s.init for reading.\n", if_basename);
		exit(1);
	}

	while ( (c = fgetc(ifp)) != EOF ) {
		if(len > 15) {
			fprintf(stderr, "Error: initial state is too large, truncating.");
		}
		tmp_data[len] = (uint8_t)c;
		len++;
	}

	return tmp_data;
}
