#include "ca_state.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>

/*
 * Writes the next 8 bytes to file pointer ofp
 */
static int
write_state(FILE* ofp, uint8_t* state)
{
	/* write the next 16 bytes to ofp */
	size_t i;
	for( i = 128; i > 0; i-- ) {
		if( bit_isset(state, i-1) )
			fprintf(ofp, "1");
		else
			fprintf(ofp, "0");
	}
	fprintf(ofp, "\n");
	return 0;
}

/*
 * Takes out_file and a pointer to a states structure
 * as input and writes the states to the output file.
 */
int
write_states(const char* out_file, ca_states_t* states)
{
	FILE* ofp;
	size_t i;
	if ( (ofp = fopen(out_file, "a+")) == NULL) {
		fprintf(stderr, "Error opening file for writing: %s", out_file);
		return 1;
	}
	for( i = 0; i < states->len; i += 16 ) {
		write_state(ofp, &states->data[i]);
	}
	return 0;
}

/*
 * Takes a string of bits as input and returns an
 * array of uint8_t with the bits.
 */
uint8_t*
str_to_state(const char* string_state)
{
	size_t i;
	uint8_t* state = malloc(sizeof(uint8_t)*16);
	if( strlen(string_state) != 128 ) {
		fprintf(stderr, "Error; string not 128 bit long, only: %zu", strlen(string_state));
	}

	for( i = 0; i < strlen(string_state); i++ ) {
		if( string_state[i] == 1 )
			bit_set(state, i);
		else if( string_state[i] == 0 )
			bit_clr(state, i);
		else
			fprintf(stderr, "Error wrong bit value 0x%x", string_state[i]);
	}
	return state;
}
