#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "emb.h"
#include "datalink.h"
#include <stdint.h>
#include <arpa/inet.h>
#include "pin.h"
#include <time.h>
#include "config.h"
#include "ca_program.h"
#include "ca_state.h"
#include <string.h>
#include <unistd.h>
#include "datalink.h"
//#define HAVE_LITTLE_ENDIAN 1
//#define HAVE_BIG_ENDIAN 0
//#include "endian.h"

/* Send initial state and 32 programs, one for each core,
 * remember the length of the data depends on the programs.
 * The payload will start be:
 * 8 bytes - initial state.
 * 1 byte - program 0 length.
 * n byte - program 0.
 * 1 byte - program 1 length.
 * m byte - program 1.
 * up to 32.
 */
ca_states_t* evo_config ( uint8_t* initial_state, ca_program_t** programs )
{
	uint32_t len = 0;
	uint8_t* data; /*payload data */
	size_t i, j, data_len=0;
	uint16_t be_len; /* big endian program size temp variable */
	int response;
	evoframe_t* frame;
	ca_states_t* states = malloc(sizeof(ca_states_t));

	/* Sum up the length of all the programs, remember to add initial state */
	for( i = 0; i < 32; i++ ) {
		len += programs[i]->len + 2; /* + 1 byte for program length */
	}
	len += 16; /* + initial state ( 16 bytes, 128 bits ) */

	data = calloc(len, sizeof(uint8_t));
	
	/* copy initial state and programs to the payload buffer */
	for( data_len = 0; data_len < 16; data_len++ ) {
		data[data_len] = initial_state[data_len];
	}
	data_len=16; /*next byte */
	for( i = 0; i < 32; i++ ) {
		be_len = htons(programs[i]->len);
		memcpy(&data[data_len], &be_len, 2);
		data_len += 2;
//		data[data_len++] = htons(programs[i]->len);
		for( j = 0; j < programs[i]->len; j++ ) {
			data[data_len++] = programs[i]->data[j];
		}
	}

	frame = create_frame(CMD_CONF_EVO, data, len);

	printf("Sending frame..len: %d\n", len);
	send_frame(frame, len);
	printf("Sent\n");
	free(frame);
	usleep(500000);

	if((response = getResponse()) != RESP_OK) {
		fprintf(stderr, "evo_config failed with code %d\n", response);
		exit(1);
	}
	
	frame = recv_frame();
	printf("Received frame: %d bytes\n",ntohl(frame->header->len_bytes));
/*	for(i=0;i<ntohl(frame->header->len_bytes);i++) {
		printf("0x%x ", frame->data[i]);
	}
	printf("\n");*/

	states->len = (size_t)ntohl(frame->header->len_bytes);
	states->data = frame->data;
//	free(frame->data);
	free(frame);
	return states;
}

int setPattern(pattern_t * pattern, pinconfig_t * conf)
{
//    printf("Sending pattern\n");
    uint8_t * data = config_to_emb_data(pattern, conf);
    evoframe_t frame;
    evoframe_header_t * head;
   
    head = create_frame_header(0x42, conf->npins/PINS_PER_BYTE, 0, CMD_SEND_PATTERN); 
 //   printf("type: %d\n", head.type);
    frame.header = head;
    frame.data = data;
    
    send_frame(&frame, conf->npins/PINS_PER_BYTE);
    free(head);

    return 0;
}


pattern_t * readPattern(pinconfig_t * config)
{
    //Send command to fpga
    evoframe_t frame;
    evoframe_header_t * head;
    head = create_frame_header(0x42, 0, 0, CMD_READ_PATTERN);
    frame.header = head;
    frame.data = NULL;
    
    send_frame(&frame, 0);
    free(head);
    int response;
    
 //   printf("Waiting for response from UC:\n");
    if((response = getResponse()) != RESP_OK) {
        fprintf(stderr, "readPattern(): send_frame() failed with code %d\n", response);
    }

    //Wait for data back.
    evoframe_t * frame_ret;
    frame_ret = recv_frame();

    //Mask out the output pins according to the config. 
    //get_input_bytes returns allocated data.
    int i;
    uint8_t * retdata = get_input_bytes(frame_ret->data, config);

    //Done with the frame data.
    free(frame_ret->data);
  
    pattern_t * ret = malloc(sizeof(pattern_t));
    ret->nbytes = (int)ceil(config->ninput/(double)8);
    ret->data = retdata; 
    
    return ret;
}

pattern_t * setWaitRead(pattern_t * pattern, pinconfig_t * config, int wait)
{
    setPattern(pattern, config);
    usleep(1000*wait);
    return readPattern(config);
}


int getResponse()
{
    evoframe_t * ret = recv_frame();

    if(ret->header->type != CMD_RESPONSE) {
        fprintf(stderr, "Expected response from EMB, got %d-type frame\n", ret->header->type);
    } else {
        //printf("Response ok!\n");
    }

    uint32_t response = ntohl(*(uint32_t*)ret->data);
    free(ret->data);

    return (int)response;
}


int getNumBytesInput(pinconfig_t * conf) 
{   
    int i;
    int n;
    for(i = 0; i < conf->npins; i++) {
        if(conf->pins[i].mode == PIN_IN) {
            n++;
        }
    }
    return n/8;
}

int getNumBytesOutput(pinconfig_t * conf) 
{   
    int i;
    int n;
    for(i = 0; i < conf->npins; i++) {
        if(conf->pins[i].mode == PIN_OUT) {
            n++;
        }
    }
    return n/8;
}


int allocatePatterns(int n, pinconfig_t * config) 
{
    uint32_t data = htonl(n);
    evoframe_t * cmd = create_frame(CMD_ALLOC, (uint8_t*)&data, sizeof(int) );
    send_frame(cmd, sizeof(int));

    free(cmd); 

    return getResponse();
}

int storePattern(pattern_t * pattern, int n, pinconfig_t * conf)
{
    uint8_t * data = config_to_emb_data(pattern, conf);
    int i;

    evoframe_t * frame = create_frame(CMD_STORE_VECTOR, data, CFG_VECTOR_BYTES);
    frame->header->vector = htonl(n);

    send_frame(frame, CFG_VECTOR_BYTES);

    return getResponse();
}

int storeRndPattern(int num, pinconfig_t * conf)
{
    //Create a random pattern and store it using storePattern();
    int pattern_bytes = (int)ceil(conf->noutput/(double)8);
    pattern_t * pat = generate_random_pattern(pattern_bytes);

    return storePattern(pat, num, conf);
}

pattern_t * applyPattern(int n, pinconfig_t * config, int wait)
{
    evoframe_t * frame = create_frame(CMD_APPLY_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    
    uint32_t data = htonl(wait);
    frame->data = (uint8_t *)&data;
    
    send_frame(frame, 0);
    if(getResponse() != 0) {
        fprintf(stderr, "applyPattern failed board-side\n");
    } else {
        printf("applyPattern succeeded\n");
    }

    frame = create_frame(CMD_READ_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    send_frame(frame, 0);
   
    printf("reading vector\n");
    return loadPattern(n, config);
}

int detectStabilityPattern(uint32_t samplePeriod, uint32_t stabilityPeriod, 
                           uint32_t timeout, pattern_t * pat, 
                           pinconfig_t * config)
{

    //First we set the pattern.
    printf("setting pattern in detectStability\n");
    setPattern(pat, config);
   
    printf("ok, done. next send command\n");
    //Then, we send a command.
    uint32_t data[4];
    data[0] = htonl(samplePeriod);
    data[1] = htonl(stabilityPeriod);
    data[2] = htonl(timeout);
    data[3] = 0x88;

    evoframe_t * frame = create_frame(CMD_DETECT_STABILITY, (uint8_t*)data, 4 * sizeof(uint32_t));

    //ship command to the uC
    send_frame(frame, 4 * sizeof(uint32_t));
     
    return getResponse();
}


pattern_t * loadPattern(int n, pinconfig_t * config)
{
    evoframe_t * frame = create_frame(CMD_READ_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    
    send_frame(frame, 0);

    //We will now get a result.
    frame = recv_frame();
    int i=0;
    uint8_t * retdata = get_input_bytes(frame->data, config);
    free(frame->data);

    pattern_t * ret = malloc(sizeof(pattern_t));
    ret->data = retdata; 
    ret->nbytes = (int)ceil(config->ninput/(double)8);

    return ret;
}

pattern_t ** applyAllPatterns(int wait, int n, pinconfig_t * cfg)
{
    uint32_t data = htonl(wait);
    evoframe_t * cmd = create_frame(CMD_APPLY_VECTOR_SET, (uint8_t*)&data, sizeof(uint32_t) );
    send_frame(cmd, sizeof(int));

    //Allocate room for return vectors 
    int pattern_bytes = (int)ceil(cfg->ninput/(double)8);

    pattern_t ** ret = malloc(sizeof(pattern_t *) * n);
    int i;
    for(i = 0; i < n; i++) {
        (ret[i]) = loadPattern(i, cfg);
    }
    
    return ret;
}

/* Generate a random pattern */
pattern_t * generate_random_pattern(int bytes)
{
    int i;
    pattern_t * pat = (pattern_t*)malloc(sizeof(pattern_t *));
    pat->nbytes = bytes;
    pat->data = malloc(bytes);

    struct timespec ts;
    clock_gettime(CLOCK_MONOTONIC, &ts);
    srand(ts.tv_nsec);
 
    //seed it with time.

    for(i = 0; i < bytes; i++) {
        pat->data[i] = rand(); 
    }
    return pat;
}

void free_patterns(pattern_t ** nbs, int n) 
{
    //Free all neighbours, one by one
    int i;
    for(i = 0; i < n; i++) {
        free(nbs[i]);            
    }
    //Free list itself
    free(nbs);
}


