#include <string.h>
#include "command.h"
#include "datalink.h"
#include "fpga_config.h"
#include "evo_debug.h"
#include "cycle_counter.h"
#include "print_funcs.h"
#include "tc.h"

#include "config.h"
#include "fpga.h"

#include "fpga_mem.h"

#include <stdlib.h>
#define smc_write_16(b,o,d) (((volatile U16 *)(b))[(o)] = (U16)(d))
volatile uint32_t time = 0;


int execute_command(evoframe_t * frame )
{
//	print_dbg("Executing command\n");
	switch(frame->header->type) {
		case CMD_FPGA_CONFIG:
			program_fpga(frame);
			respond(RESP_OK); 

			break;
		case CMD_SEND_PATTERN:
			//print_dbg("Writing pins\n");
			write_pin_config(frame);
			break; 
		
		case CMD_READ_PINS:
   //		 print_dbg("Reading pins\n");
			read_pin_values();
			break;

		case CMD_ALLOC:
  //		  print_dbg("Allocing\n");
			alloc_vectors(frame);
			break;

		case CMD_STORE_VECTOR:
 //		   print_dbg("Setting vector\n");
			store_vector(frame);
			break;
	
		case CMD_APPLY_VECTOR:
//			print_dbg("Applying vector\n");
			cmd_apply_vector(frame);
			break;

		 case CMD_READ_VECTOR:
	//		print_dbg("Reading vector\n");
			read_vector(frame);
			break;

		 case CMD_DETECT_STABILITY:
   //		 print_dbg("Detecting stability\n");
			detect_stability(frame);
			break;

		 case CMD_APPLY_VECTOR_SET:
  //		  print_dbg("Applying set of vectors\n");
			cmd_apply_vector_set(frame);
			break;

	case CMD_CONF_EVO:
		evo_config(frame);
		break;

		default: 
			print_dbg("Not a proper command :< ("); 
			//print_dbg_hex(frame->header->type);
			//print_dbg(")\n");
			break;
	}

	return 0;
}

void
core_program(uint8_t *data, uint32_t length)
{
	uint32_t nread = 0;
	uint32_t i;
	uint16_t n_inst;
	uint8_t corenum = 0;

	while(nread < length) {
		n_inst = (data[nread]<<8u)|(data[nread+1]); //cannot acces unaligned 16 bit values (ld.h)
		nread += 2;
		fpga_mem_copyto(core_offset(corenum)*2, data+nread, n_inst);
		sleep(50000);
		corenum++;
		nread += n_inst;
	}
}

int evo_config(evoframe_t *cmd)
{
	size_t i, j;
	volatile uint16_t *fpga = (uint16_t *)FPGA_ADDR;
	uint16_t *program_ptr;
	uint16_t program_len;
	respond(RESP_OK); //we received the command

	evoframe_t frame;
	evoframe_header_t header;
	header.marker = HEADER_MARKER;
	header.type = CMD_CONF_EVO;	
	header.len_bytes = 4096; /* all 256 ca states */ 
	frame.header = &header;
	frame.data = (uint8_t*)calloc(4096, sizeof(uint8_t));

	//Write initial config to the FPGA
	fpga_mem_copyto(PINCFG_OFFSET*2, cmd->data, 16);
	//Copy programs
	core_program(cmd->data+16, cmd->header->len_bytes);
	
	for(i=0;i<32;i++) {
		//fpga_mem_writeh(core_offset(i), (((128+i)<<8)&0xFF00)|((64)&0xFF));
	//	uint16_t t = (((128+i))&0xFF)|((64<<8)&0xFF00);
	//	fpga_mem_writeh(core_offset(i), t);
	}
	//start
	fpga_mem_writeh(CMD_OFFSET, BIG_ENDIAN(CMD_CONF_EVO));
//	char *garbage = "\xAA\x55";
	//for(j=0;j<32;j++) {
	//for(i=0;i<16;i++) {
	//	fpga_mem_copyto((core_offset(j)+i)*2, garbage, 2);
	//}
	//}
//	for(i=0;i<512;i++) {
//		fpga_mem_copyto(i*2, garbage, 2);
//	}
//	for(i=0;i<32;i++) {
		//fpga_mem_copyto(core_offset(i)*2, garbage, 32);
//		fpga_mem_copyto((PINCFG_OFFSET+i*16)*2, garbage, 32);
		//fpga_mem_copyto(core_offset(i)*2, garbage, 32);
//		for(j=0;j<16;j++) {
//			fpga_mem_writeh(core_offset(i)+j, ((uint16_t*)garbage)[j]);
//		}
//	}
	/* Write inital config to the FPGA */
	/* the 16 first bytes/8 first words is the initial state */
//	for ( i = 0; i < 8; i++ ) {
//		smc_write_16(&fpga[PINCFG_OFFSET],i,((uint16_t*)frame.data)[i]);
		//fpga[PINCFG_OFFSET + i] = ((uint16_t*)frame.data)[i];
//	}

	/* skip initial configuration, program_ptr now points to length of first program */
//	program_ptr = (uint16_t*)&frame.data[16]; 

	/* Write programs into CA cores address space */
//	for ( i = 0; i < 32; i++ ) {
//		program_len = program_ptr[0];
//		program_ptr++; /* point to first instruction word */
	//	memcpy(&fpga[CA_PROGRAMS_OFFSET + ( i * 16 )], program_ptr, program_len);
//	}

	/* start the CA */
//	fpga[CMD_OFFSET] = CMD_CONF_EVO;
//	uint16_t *d = malloc(sizeof(uint16_t)*8);
//	for ( i = 0; i < 8; i++ ) {
	//d[i]=fpga[PINCFG_OFFSET+i];
//	}
//	frame.data = (uint8_t*)d;
//	for ( i=0; i<8; i++) {
//		((uint16_t*)frame.data)[i] = fpga[PINCFG_OFFSET+i];
//	}
//	char *s = "AABBCCDDEEFFGGHHIIJJKKLLMMNNOOPPQQRRSSTTUUVVWWXXYYZZ";
//	char s2[54];
//	s2[0] = 0x1;
//	uint8_t success = 1;
//	memcpy(&fpga[PINCFG_OFFSET], s, 52);
//	fpga_mem_writeh(PINCFG_OFFSET, 0xF1F1);
//	fpga_mem_writeh(PINCFG_OFFSET+26, 0xF1F1);
//	fpga_mem_copyto(PINCFG_OFFSET*2+1, s, 52);

//	memcpy(&s2[0], ((uint8_t* )fpga)+PINCFG_OFFSET*2+1, 51);
//	volatile uint8_t* tmp_fpga =(volatile uint8_t*)&fpga[PINCFG_OFFSET];
//	fpga_mem_copyfrom(s2, PINCFG_OFFSET*2+1, 52);
//	for(i=0;i<54;i++) {
//		frame.data[i] = s2[i];
//	}

	/* wait until the FPGA is done and then send all the states back to host */
	while(fpga_mem_readh(CMD_OFFSET) != CMD_NOOP);
	//States ready copy to frame.data
//	fpga_mem_copyfrom(frame.data, 0, 32);
//	fpga_mem_copyfrom(frame.data, core_offset(0)*2, 32*32);
	fpga_mem_copyfrom(frame.data, CA_STATES_OFFSET*2, 4096);
//	char *gar = "\xFF\x55";
//	fpga_mem_copyto(PINCFG_OFFSET*2, gar, 2);
//	uint16_t t = fpga_mem_readh(PINCFG_OFFSET);
//	frame.data[1] = (t>>8)&0xFF;
//	frame.data[0] = t&0xFF;
//	uint16_t t2 = 11;
//	*((uint16_t*)frame.data) = 11;
	
//	for(i=0;i<32;i++) {
//		for(j=0;j<16;j++) {
//		((uint16_t*)frame.data)[i*16+j] = fpga_mem_readh(core_offset(i)+j);
//		}
//	}
	for(i=0;i<32;i++) {
//	}
//		uint16_t t = fpga_mem_readh(core_offset(i));
//		frame.data[i*32+1] = (t>>8)&0xFF;
//		frame.data[i*32] = t&0xFF;
	}
//		fpga_mem_copyfrom(frame.data+16, 0, 2);
//		frame.data[32] = fpga_mem_readb(0);
//		frame.data[33] = fpga_mem_readb(1);
//		fpga_mem_writeb(0, 16);
//		fpga_mem_copyfrom(frame.data, 0, 32*32);
//		t = fpga_mem_readh(0);
//		frame.data[64] = (t>>8)&0xFF;
//		frame.data[65] = t&0xFF;
	//	*((uint16_t*)&frame.data[i*2]) = fpga_mem_readh(core_offset(i));
/*	for(i=0;i<32;i++) {
		for(j=0;j<32;j++) {
			if(frame.data[i*32+j] != garbage[j]) {
				frame.data[i*32+j] = "X";
			}
		}
	}*//*
	uint16_t fpga_freq1 = fpga_mem_readh(0xB);
	uint32_t cycl1 = __builtin_mfsr(AVR32_COUNT);
	uint16_t fpga_freq2 = fpga_mem_readh(0xB);
	uint32_t cycl2 = __builtin_mfsr(AVR32_COUNT);
//	*((uint32_t*)frame.data) = (cycl2 - cycl1);
//	*(((uint32_t*)frame.data)+1) = fpga_freq;
	frame.data[0] = (fpga_freq1>>8)&0xFF;
	frame.data[1] = (fpga_freq1&0xFF);

	frame.data[16] = (fpga_freq2>>8)&0xFF;
	frame.data[17] = (fpga_freq2&0xFF);

	frame.data[32] = (cycl1>>24)&0xFF;
	frame.data[33] = (cycl1>>16)&0xFF;
	frame.data[34] = (cycl1>>8)&0xFF;
	frame.data[35] = (cycl1&0xFF);
	
	frame.data[48] = (cycl2>>24)&0xFF;
	frame.data[49] = (cycl2>>16)&0xFF;
	frame.data[50] = (cycl2>>8)&0xFF;
	frame.data[51] = (cycl2&0xFF);*/
//	}
//	memcpy(frame.data, cmd->data, cmd->header->len_bytes);
//	for(i=0;i<4096/2;i++) {
//		((uint16_t*)frame.data)[i] = fpga_mem_readh(CA_STATES_OFFSET+i);
//	}
	send_frame(&frame, 4096); //send states
	free(frame.data);
	return 0;
}

int
write_pin_config(evoframe_t * cmd)
{
	fpga_mem_copyto(PINCFG_OFFSET*2, cmd->data, 16); //copy initial config into memory
	fpga_mem_writeh(CMD_OFFSET, BIG_ENDIAN(CMD_SEND_PATTERN)); //apply the configuration to the material
	respond(RESP_OK); // we got the command	and executed it
	return 0;
}


int
read_pin_values()
{
	evoframe_t frame;
	evoframe_header_t header;
	header.marker = HEADER_MARKER;
	header.type = CMD_READ_PINS;	
	header.len_bytes = 8 * sizeof(uint16_t); 
	frame.header = &header;
	frame.data = malloc(8 * sizeof(uint16_t));
	
	//command the FPGA to write the pin values to the shmem
	fpga_mem_writeh(CMD_OFFSET, BIG_ENDIAN(CMD_READ_PINS));
 
	//Respond back that we got the command
	respond(RESP_OK); 
	
	//wait until finished
	while(fpga_mem_readh(CMD_OFFSET) != CMD_NOOP);
	fpga_mem_copyfrom(frame.data, CA_STATES_OFFSET*2, 16);
   
	//ship the frame to the host
	send_frame(&frame, frame.header->len_bytes);
	//Deallocate the frame data after use.
	free(frame.data);
	return 0;
}


void respond(int code)
{
	evoframe_t frame;
	evoframe_header_t header;
	header.marker = HEADER_MARKER;
	header.len_bytes = 4;
	header.seq = 0x32;
	header.pid = 0;
	header.type = CMD_RESPONSE;
	frame.header = &header;
	uint8_t * data = (uint8_t*)&code;
	frame.data = data;
	send_frame(&frame, 4);
} 

int alloc_vectors(evoframe_t * frame) 
{
	num_vectors = *((uint32_t*)frame->data);
	int i; 
	configs = malloc(sizeof(uint8_t * ) * num_vectors);
	responses = calloc(num_vectors, sizeof(uint8_t *));
	int failed;

	for(i = 0; i < num_vectors; i++) {
		configs[i] = malloc(CFG_VECTOR_BYTES);
		responses[i] = calloc(sizeof(uint8_t), RESPONSE_BYTES);
		if(configs[i] == NULL || responses[i] == NULL) {
			failed = 1;
		 }
	}

	if(failed) {
		respond(RESP_ALLOC_FAILED);
	} else {
		respond(RESP_OK);
	}

	return 0;
}

int store_vector(evoframe_t * frame)
{
	uint32_t n = frame->header->vector; 
//	print_dbg("Setting vector ");
 //   print_dbg_hex(n);
  //  print_dbg("\n");
  
	//Copy the frame data into the config memory.
	//The frame will be freed later on.
	memcpy(configs[n], frame->data, frame->header->len_bytes);

	/*
	print_dbg("\n\nConfig written to vector memory:\n");

	uint16_t bank;
	for(bank = 0; bank < 8; bank++) {
		 print_dbg_hex(*((uint16_t *)configs[n] + bank));
		 print_dbg("\n");
	}
	*/

	respond(RESP_OK);
	return 0;
}



int cmd_apply_vector(evoframe_t * frame)
{
	int n = frame->header->vector;
	uint32_t wait = *((uint32_t*)frame->data);
	
	//The config data is stored on board, just apply this vector.
	apply_vector((uint16_t *)(configs[n]), wait, responses[n]);

	respond(RESP_OK);

	return 0;
}

int read_vector(evoframe_t * frame) 
{
	int n = frame->header->vector;
	uint8_t * bytes = (responses[n]);

	evoframe_t ret;
	evoframe_header_t header;
	header.marker = HEADER_MARKER;
	header.len_bytes = RESPONSE_BYTES ;
	ret.header = &header;
	ret.data = bytes;
   
	send_frame(&ret, RESPONSE_BYTES); 

	return 0;
}


int detect_stability(evoframe_t * frame)
{
	
	uint32_t sample_period; // = 0x200;
	uint32_t timeout; // = 0x1000;
	uint32_t stable; // = 0x500;
   
	uint32_t * daten = (uint32_t*)frame->data;
	memcpy(&sample_period, &(daten[0]), 4);
	memcpy(&stable, &(daten[1]), 4);
	memcpy(&timeout, &(daten[2]), 4);
	print_dbg("magic byte:");
	print_dbg_hex(daten[3]);
	print_dbg("\n");

	print_dbg("detecting stability, period \n");
	print_dbg_hex(sample_period);
	print_dbg(", timeout ");
	print_dbg_hex(timeout);
	print_dbg(", stable ");
	print_dbg_hex(stable);
	print_dbg("\n");

	uint16_t * new_values = malloc(CFG_VECTOR_BYTES);
	uint16_t * last_values = malloc(CFG_VECTOR_BYTES);

		//We idle here until we're stable or we have a timeout.
	//If we've been inside the interrupt routine and
	//fetched new values, we iterate over the fetched
	//values and compare them to what we had last time.

   
	int last_time = 0;
	int delta_time = 0;
	int stable_time = 0;
	int stability_detected = 0;
	int timeout_reached = 0;
	int last_period_start = 0;

	//uint16_t * fpga = (uint16_t * )FPGA_ADDR;

	int i;
   
	//Reset time! 
	time = 0;
	
	while(!stability_detected || !timeout_reached) {
		delta_time = (time - last_time);
		last_time = time;
	  
		//Read new values. 
		if((time - last_period_start) > sample_period) {
			last_period_start = time;
/*			for(i = 0; i < NBANKS; i++) {
				fpga[CMD_OFFSET] = CMD_READ_PINS;
				new_values[i] = fpga[PINVAL_OFFSET + i];
			}*/
		fpga_mem_copyfrom(new_values, PINVAL_OFFSET*2, 16); // converted to new fpga memory api

			if(equal_values(last_values, new_values, NBANKS)) {
				stable_time += sample_period;
				//print_dbg("stable time:");
				//print_dbg_hex(stable_time);
				//print_dbg("\n");
			} else {
				stable_time = 0;
			}

			//Flip pointers to arrays. 
			uint16_t * tmp = last_values;
			last_values = new_values;
			new_values = tmp;
		}
			
		//Check if we should terminate loop now.
		if(stable_time >= stable) {
			stability_detected = 1;
		 //   break;
		}
		
		if(time >= timeout) {
			timeout_reached = 1;
		  //  break;
		}
	}
  
	//Stop the clock. 
	//tc_stop(tc, TC_CHANNEL);

	free(last_values);
	free(new_values);

	//If we exited with a stable configuration. 
	if(stability_detected) {
		print_dbg("Stability\n");
		respond(RESP_OK);
		return 0;
	}

	//We timed out, let the world know.
	if(timeout_reached) {
		print_dbg("timeout reached\n");
		respond(RESP_TIMEOUT);
		return 0;
	}

	respond(RESP_OK);
	return 0;
}

int cmd_apply_vector_set(evoframe_t * frame)
{
	int wait = *((uint32_t *)frame->data);
	int i;
	print_dbg("time: ");
	print_dbg_hex(time);
	for(i = 0; i < num_vectors; i++) {
		apply_vector((uint16_t*)configs[i], wait, responses[i]);
	}
	print_dbg("\n time after:");
	print_dbg_hex(time);
	print_dbg("\n");
	return 0;
}

__attribute__((__interrupt__)) void stability_read(void)
{
	time++;
	tc_read_sr(&(AVR32_TC), TC_CHANNEL);
}


int equal_values(uint16_t * a, uint16_t * b, int n)
{
	int i;
	for(i = 0; i < n; i++) {
		if(a[i] != b[i]) {
			return 0;
		}
	}
	print_dbg("equal\n");
	return 1;
}



