#include "fpga_mem.h"
uint8_t
fpga_mem_readb(uint16_t byte_offset)
{
	uint16_t tmp = fpga_mem_readh(byte_offset/2);
	return (byte_offset & 1) ? (tmp & 0xFF) : ((tmp>>8) & 0xFF);
}

void
fpga_mem_writeb(uint16_t byte_offset, uint8_t data)
{
	uint16_t tmp = fpga_mem_readh(byte_offset/2);
	uint16_t hword = (byte_offset & 1) ?
			(tmp & 0xFF00) | ((uint16_t) data):
			(tmp & 0xFF) | (((uint16_t) data)<<8);
	fpga_mem_writeh(byte_offset/2, hword);
}

void
fpga_mem_copyfrom(void *dest, uint16_t byte_offset, uint16_t nbytes)
{
	uint8_t *dest_ptr = dest; 
	uint16_t nread = 0;

	if((byte_offset & 1)) {
		dest_ptr[nread] = fpga_mem_readb(byte_offset);
		nread++;
	}
	for(;nread+1 < nbytes; nread += 2) {
		uint16_t tmp = fpga_mem_readh((byte_offset+nread)/2);
		dest_ptr[nread] = (tmp>>8) & 0xFF;
		dest_ptr[nread+1] = tmp & 0xFF;
	}
	if(nread<nbytes) {
		dest_ptr[nread] = fpga_mem_readb(byte_offset+nread);
	}
}

void
fpga_mem_copyto(uint16_t byte_offset, void *src, uint16_t nbytes)
{
	uint8_t *src_ptr = src;
	uint16_t nwritten = 0;

	if((byte_offset & 1)) {
		fpga_mem_writeb(byte_offset, src_ptr[nwritten]);
		nwritten++;
	}
	for(;nwritten+1 < nbytes; nwritten += 2) {
		uint16_t tmp = (((((uint16_t)src_ptr[nwritten]))<<8)&0xFF00) | (((uint16_t)src_ptr[nwritten+1])&0xFF);
		fpga_mem_writeh((byte_offset+nwritten)/2, tmp);
	}

	if(nwritten<nbytes) {
		fpga_mem_writeb(byte_offset+nwritten, src_ptr[nwritten]);
	}
}
/*
fpga_mem_readw
fpga_mem_writew
*/
